@@ -10,7 +10,7 @@ use IO::Async::Loop;
 use Net::Async::WebSocket::Client;
 Net::Async::WebSocket::Client->VERSION(0.12); # fixes some errors with masked frames
 
-our $VERSION = '0.41';
+our $VERSION = '0.42';
 
 =head1 NAME
 
@@ -52,7 +52,9 @@ sub connect( $self, $handler, $got_endpoint, $logger ) {
             # Kick off the continous polling
             on_frame => sub {
                 my( $connection, $message )=@_;
-                $handler->on_response( $connection, $message )
+                if( $handler ) { # may have gone away already
+                    $handler->on_response( $connection, $message )
+                };
             },
             on_read_eof => sub {
                 my( $connection )=@_;
